
      module module_wrfchem_lib

      implicit none

!     include files
      include 'netcdf.inc'

!     public  procedures
      public  :: init_wrfchem_lib, exit_wrfchem_lib
      public  :: wrfchem_readscalar, wrfchem_read2d,  wrfchem_read3d
!tks     public  :: wrfchem_write4d
      public  :: wrfchem_write4d_bc, wrfchem_write4d_ic

!     private procedures
      private :: handle_error, define_variable_bc,define_variable_ic

!     public variables
      integer, public  :: nx, ny, nz, nw, ntime      ! data dimension length
      integer, public  :: nx_ic, ny_ic, nz_ic, ntime_ic ! data dimension length
      integer, private :: nscalar, nstring           ! data dimension length
      integer, public  :: year_start, month_start, day_start
      integer, public  :: hour_start, minute_start, second_start
      integer, public  :: year_end, month_end, day_end
      integer, public  :: hour_end, minute_end, second_end
      integer, public  :: dt                        ! temporal resolution

!     variables for reading netCDF data
      integer          :: ncid, ncidi                          ! netCDF file IDs
      integer, private :: nstt(4), ncnt(4), ncntx(4), ncnty(4) ! start index and counter array

      contains

!---------------------------------------------------------------------------------------------------
!     initialize netcdf file
!---------------------------------------------------------------------------------------------------

!tks     subroutine init_wrfchem_lib( wrfchem_bdy_fn, wrfchem_input_fn, specname, nspec, ntime )
      subroutine init_wrfchem_lib( wrfchem_bdy_fn, wrfchem_input_fn, specname, nspec, ntime, do_bc, do_ic )

      implicit none

!     input arguments
      integer, intent(in) :: nspec
!tks
      logical, intent(in) :: do_bc
      logical, intent(in) :: do_ic
!tks
      integer, intent(out) :: ntime
      character(len=*), intent(in) ::  wrfchem_bdy_fn, wrfchem_input_fn
      character(len=12), intent(in), DIMENSION(nspec) :: specname

!     local argumets
      integer :: status
      integer :: xdimid,  ydimid, zdimid, wdimid, tdimid, sdimid
      integer :: ndims, dimids(4)
      integer :: vid
      integer :: ip, n, ns, i
      integer :: year_first, month_first, day_first
      integer :: hour_first, minute_first, second_first
      character(len=100) :: dtstrings, dtstringe, dtstringf
      character(len=20)  :: spec1
      character(len=3)   :: order



!     open the input NETCDF file
      write(6,*)'wrfchem_input_fn',wrfchem_input_fn
      status = nf_open( wrfchem_input_fn, nf_write, ncidi )
      if( status /= nf_noerr ) call handle_error( status )

! tks initial condition file
      if ( do_ic ) then
         status = nf_inq_dimid( ncidi, 'west_east', xdimid )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimlen( ncidi, xdimid, nx_ic )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimid( ncidi, 'south_north', ydimid )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimlen( ncidi, ydimid, ny_ic )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimid( ncidi, 'bottom_top', zdimid )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimlen( ncidi, zdimid, nz_ic )
         if( status /= nf_noerr ) call handle_error( status )

         status = nf_inq_dimid( ncidi, 'Time', tdimid )
         if( status /= nf_noerr )  call handle_error( status )
         status = nf_inq_dimlen( ncidi, tdimid, ntime_ic )
         if( status /= nf_noerr ) call handle_error( status )

         !     enter redefine mode
         status = nf_redef( ncidi )
         if( status /= nf_noerr ) call handle_error( status )

         !     define variables
         ndims = 4
         ! west-east
         dimids = (/ xdimid, ydimid, zdimid, tdimid /)

         do ns = 1, nspec
            status = nf_inq_varid( ncidi, trim(specname(ns)), vid )
            if( status /= nf_noerr ) then
               print*, 'Define new species for IC : ', trim(specname(ns))
               order = 'XYZ'
               call define_variable_ic( trim(specname(ns)), nf_real, ndims, dimids, order )
            else
!              print*, 'Overwriting species for IC : ', trim(specname(ns))
               if(ns.eq.1)print*, 'Overwriting species for IC : '
            endif
         end do

         !     end define mode
         status = nf_enddef( ncidi )
         if( status /= nf_noerr ) call handle_error( status )
         print*,'DONE WITH IC'
      end if   !end ic
!tks
!     open the bdy NETCDF file
        write(6,*)'open wrfchem_bdy_fn',wrfchem_bdy_fn
         status = nf_open( wrfchem_bdy_fn, nf_write, ncid )

      if( status /= nf_noerr ) call handle_error( status )

!     get spacial dimension lengths
      status = nf_inq_dimid( ncid, 'west_east', xdimid )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimlen( ncid, xdimid, nx )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimid( ncid, 'south_north', ydimid )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimlen( ncid, ydimid, ny )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimid( ncid, 'bottom_top', zdimid )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimlen( ncid, zdimid, nz )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimid( ncid, 'bdy_width', wdimid )
      if( status /= nf_noerr ) call handle_error( status )

      status = nf_inq_dimlen( ncid, wdimid, nw )
      if( status /= nf_noerr ) call handle_error( status )

!     get the time dimension length
      status = nf_inq_dimid( ncid, 'Time', tdimid )
      if( status /= nf_noerr )  call handle_error( status )

      status = nf_inq_dimlen( ncid, tdimid, ntime )
      if( status /= nf_noerr ) call handle_error( status )

!     get the string dimension length
      status = nf_inq_dimid( ncid, 'DateStrLen', sdimid )
      if( status /= nf_noerr )  call handle_error( status )

      status = nf_inq_dimlen( ncid, sdimid, nstring )
      if( status /= nf_noerr ) call handle_error( status )
      write(6,*)'nstring',nstring

!! for boundary condition
!tks
    if( do_bc ) then

!     enter redefine mode
      status = nf_redef( ncid )
      if( status /= nf_noerr ) call handle_error( status )

!     define variables      
      ndims = 4
      
      ! west-east
      dimids(1:4) = (/ ydimid, zdimid, wdimid, tdimid /)
      ! west
      do i = 1, 2
         do ns = 1, nspec
!           print*,'specname ',specname(ns)
            if(i==1) spec1 = trim(specname(ns))//'_BXS'
            if(i==2) spec1 = trim(specname(ns))//'_BTXS'
            status = nf_inq_varid( ncid, trim(spec1), vid )
            if( status /= nf_noerr ) then 
               print*, 'Define new species : ', trim(spec1)
               order = 'YSZ'
               call define_variable_bc( trim(spec1), nf_real, ndims, dimids, order )
            endif
         enddo
      end do

      ! east
      do i = 1, 2 
         do ns = 1, nspec
         if(i==1) spec1 = trim(specname(ns))//'_BXE'
         if(i==2) spec1 = trim(specname(ns))//'_BTXE'
            status = nf_inq_varid( ncid, trim(spec1), vid )
            if( status /= nf_noerr ) then 
               print*, 'Define new species : ', trim(spec1)
               order = 'YEZ'
               call define_variable_bc( trim(spec1), nf_real, ndims, dimids, order )
            endif
         enddo
      end do
      
      ! north-south
      dimids(1:4) = (/ xdimid, zdimid, wdimid, tdimid /)
      ! north
      do i = 1, 2 
         do ns = 1, nspec
         if(i==1) spec1 = trim(specname(ns))//'_BYS'
         if(i==2) spec1 = trim(specname(ns))//'_BTYS'
            status = nf_inq_varid( ncid, trim(spec1), vid )
            if( status /= nf_noerr ) then 
               print*, 'Define new species : ', trim(spec1)
               order = 'XEZ'
               call define_variable_bc( trim(spec1), nf_real, ndims, dimids, order )
            endif
         enddo
      end do

      ! south
      do i = 1, 2 
         do ns = 1, nspec
         if(i==1) spec1 = trim(specname(ns))//'_BYE'
         if(i==2) spec1 = trim(specname(ns))//'_BTYE'
            status = nf_inq_varid( ncid, trim(spec1), vid )
            if( status /= nf_noerr ) then 
               print*, 'Define new species : ', trim(spec1)
               order = 'XSZ'
               call define_variable_bc( trim(spec1), nf_real, ndims, dimids, order )
            endif
         enddo
      end do
 

!     end define mode
      status = nf_enddef( ncid )
      if( status /= nf_noerr ) call handle_error( status )

      print*,'DONE WITH BC'
   endif            !tks

!     read start and end date and second time stamp
      status = nf_inq_varid( ncid, 'Times', vid )
      if( status /= nf_noerr )  call handle_error( status )

      print*,'hello1'
      write(6,*)'ntime',ntime


      nstt(1:2) = (/ 1, 1 /)
      ncnt(1:2) = (/ nstring, 1 /)
      status = nf_get_vara_text( ncid, vid, nstt(1:2), ncnt(1:2), dtstrings )

      nstt(1:2) = (/ 1, ntime /)
      status = nf_get_vara_text( ncid, vid, nstt(1:2), ncnt(1:2), dtstringe )
      if(ntime>1)then
      nstt(1:2) = (/ 1, 2 /)
      write(6,*)'nstt',nstt(1:2),'ncnt',ncnt(1:2)
      status = nf_get_vara_text( ncid, vid, nstt(1:2), ncnt(1:2), dtstringf )
      if( status /= nf_noerr )  call handle_error( status )
      else
        write(6,*)'dtstrings',trim(dtstrings)
        write(6,*)'dtstringe',trim(dtstringe)
        dtstringf=dtstringe
      endif

!     get the location of '_'
      ip = index( dtstrings, '_' )

!     delete '-', '_' and ':'
      do n = 1, nstring 
        if( (dtstrings(n:n)=='-') .or. (dtstrings(n:n)=='_') .or.  &
             dtstrings(n:n)==':' ) then
          dtstrings(n:n) = ' '
        endif
        if( (dtstringe(n:n)=='-') .or. (dtstringe(n:n)=='_') .or.  &
             dtstringe(n:n)==':' ) then
          dtstringe(n:n) = ' '
        endif
        if( (dtstringf(n:n)=='-') .or. (dtstringf(n:n)=='_') .or.  &
             dtstringf(n:n)==':' ) then
          dtstringf(n:n) = ' '
        endif
      enddo

      print*,'hello2'
!     read year, month and day
      read( dtstrings(:ip), * ) year_start, month_start, day_start
      print*,'hello2a', year_start, month_start, day_start
      read( dtstrings(ip:nstring), * ) hour_start, minute_start, second_start
      print*,'hello2a', hour_start, minute_start, second_start

      print*,'hello2b'

      read( dtstringe(:ip), * ) year_end, month_end, day_end
      print*,'hello2bb'
      read( dtstringe(ip:nstring), * ) hour_end, minute_end, second_end
      print*,'hello2c'


      print*,'hello3'
      write(6,*)'ip',ip
      write(6,*)'dtstringf',dtstringf(:ip)

      read( dtstringf(:ip), * ) year_first, month_first, day_first
      write(6,*)'year_first',year_first, month_first, day_first
      call flush(6)
      write(6,*)'nstring',nstring
      write(6,*)'dtstringf',dtstringf(ip:nstring)
      call flush(6)


      read( dtstringf(ip:nstring), * ) hour_first, minute_first, second_first
      write(6,*)'hour_first',hour_first, minute_first, second_first

      ! calculate temporal resolution 
      if(hour_first == 0 .and. hour_start >= 12) hour_first = 24
      if(hour_start == 0 .and. hour_first >= 12) hour_start = 24
      dt = abs(hour_first - hour_start)
      
      write(6,*) 'succesfully init module_wrfchem_lib ...'
      call flush(6)


      end subroutine init_wrfchem_lib

!---------------------------------------------------------------------------------------------------
!     define variable
!     1. Define variable according input arguments
!     2. Define the attributes of the variable
!---------------------------------------------------------------------------------------------------

      subroutine define_variable_bc( vname, nf_type, ndims, dimids, order )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname
      integer, intent(in)           :: nf_type
      integer, intent(in)           :: ndims
      integer, intent(in)           :: dimids(ndims)
      character (len=*), intent(in) :: order

!     local arguments
      integer :: status
      character(len=20) :: string 
      integer :: vid

!     define variables
      status = nf_def_var( ncid, vname, nf_type, ndims, dimids, vid )
      if( status /= nf_noerr )  call handle_error( status )

!     define attributes
      status = nf_put_att_int( ncid, vid, 'FieldType', nf_int, 1, 104 )
      if( status /= nf_noerr )  call handle_error( status )

      string = trim(order) 
      status = nf_put_att_text( ncid, vid, 'MemoryOrder', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = '-' 
      status = nf_put_att_text( ncid, vid, 'description', len_trim(string), trim(string) ) 
      if( status /= nf_noerr )  call handle_error( status )

      string = '-'
      status = nf_put_att_text( ncid, vid, 'units', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = ' ' 
      status = nf_put_att_text( ncid, vid, 'stagger', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      end subroutine define_variable_bc

!---------------------------------------------------------------------------------------------------
!     define variable
!     1. Define variable according input arguments
!     2. Define the attributes of the variable
!---------------------------------------------------------------------------------------------------
      subroutine define_variable_ic( vname, nf_type, ndims, dimids, order )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname
      integer, intent(in)           :: nf_type
      integer, intent(in)           :: ndims
      integer, intent(in)           :: dimids(ndims)
      character (len=*), intent(in) :: order

!     local arguments
      integer :: status
      character(len=20) :: string
      integer :: vid

!     define variables
      status = nf_def_var( ncidi, vname, nf_type, ndims, dimids(1:3), vid )
      if( status /= nf_noerr )  call handle_error( status )

!     define attributes
      status = nf_put_att_int( ncidi, vid, 'FieldType', nf_int, 1, 104 )
      if( status /= nf_noerr )  call handle_error( status )

      string = trim(order)
      status = nf_put_att_text( ncidi, vid, 'MemoryOrder', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = 'Volume mixing ratio'
      status = nf_put_att_text( ncidi, vid, 'description', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = 'ppm'
      status = nf_put_att_text( ncidi, vid, 'units', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = ' '
      status = nf_put_att_text( ncidi, vid, 'stagger', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      string = 'XLONG XLAT'
      status = nf_put_att_text( ncidi, vid, 'coordinates', len_trim(string), trim(string) )
      if( status /= nf_noerr )  call handle_error( status )

      end subroutine define_variable_ic
!---------------------------------------------------------------------------------------------------
!     read scalar data 
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_readscalar( vname, val )   ! read scalar variable 

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname

!     output arugments
      real, intent(out)        :: val

!     local arguments
      integer :: status
      integer :: varid

!     get variable ID, if no such a variable, return
      status = nf_inq_varid( ncidi, vname, varid )
      if( status /= nf_noerr ) call handle_error( status )
!     print *,' wrfchem_readscalar: get ',vname,varid

!     read value
      nstt(1:2) = (/ 1, 1 /)
      ncnt(1:2) = (/ 1, 1 /)
      status = nf_get_vara_real( ncidi, varid, nstt(1:2), ncnt(1:2), val )
      if( status /= nf_noerr ) call handle_error( status )

!     print *,' wrfchem_readscalar: value ',val,status

      end subroutine wrfchem_readscalar

!---------------------------------------------------------------------------------------------------
!     read integer data 
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_readinteger( vname, val )   ! read integer variable 

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname

!     output arugments
      integer, intent(out)        :: val

!     local arguments
      integer :: status
      integer :: varid

!     get variable ID, if no such a variable, return
      status = nf_inq_varid( ncidi, vname, varid )
      if( status /= nf_noerr ) call handle_error( status )
      print *,' wrfchem_readint: get ',vname,varid,ncidi

!     read value
      nstt(1:2) = (/ 1, 1 /)
      ncnt(1:2) = (/ 1, 1 /)
      status = nf_get_vara_int( ncidi, varid, nstt(1:2), ncnt(1:2), val )
      if( status /= nf_noerr ) call handle_error( status )

      print *,' wrfchem_readint: value ',val,status

      end subroutine wrfchem_readinteger

!---------------------------------------------------------------------------------------------------
!     read one-dimensional data
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_read2d( vname, val )   ! read 2d variable( a slice of data )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname

!     output arugments
      real, intent(out)        :: val(nz)

!     local arguments
      integer :: status
      integer :: varid

!     get variable ID, if no such a variable, return
      status = nf_inq_varid( ncidi, vname, varid )
      if( status /= nf_noerr ) call handle_error( status )

!     read value
      nstt(1:2) = (/ 1, 1 /)
      ncnt(1:2) = (/ nz, 1 /)
      status = nf_get_vara_real( ncidi, varid, nstt(1:2), ncnt(1:2), val(:) )
      if( status /= nf_noerr ) call handle_error( status )

      end subroutine wrfchem_read2d

!---------------------------------------------------------------------------------------------------
!     read three-dimensional data
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_read3d( vname, val )   ! read 3d variable( a slice of data )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname

!     output arugments
      real, intent(out)        :: val(nx,ny)

!     local arguments
      integer :: status
      integer :: varid

!     get variable ID, if no such a variable, return
      status = nf_inq_varid( ncidi, vname, varid )
      if( status /= nf_noerr ) call handle_error( status )

!     read value
      nstt(1:3) = (/ 1, 1, 1 /)
      ncnt(1:3) = (/ nx, ny, 1 /)
      status = nf_get_vara_real( ncidi, varid, nstt(1:3), ncnt(1:3), val(:,:) )
      if( status /= nf_noerr ) call handle_error( status )

      end subroutine wrfchem_read3d

!---------------------------------------------------------------------------------------------------
!     write four-dimensional data
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_write4d_bc( vname, valxs, valxe, valys, valye, vatxs, vatxe, vatys, vatye,it )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname
      real, intent(in)         :: valxs(ny,nz,nw)
      real, intent(in)         :: valxe(ny,nz,nw)
      real, intent(in)         :: valys(nx,nz,nw)
      real, intent(in)         :: valye(nx,nz,nw)
      real, intent(in)         :: vatxs(ny,nz,nw)
      real, intent(in)         :: vatxe(ny,nz,nw)
      real, intent(in)         :: vatys(nx,nz,nw)
      real, intent(in)         :: vatye(nx,nz,nw)
      integer, intent(in)      :: it

!     output arugments

!     local arguments
      integer :: status
      integer :: varid
      integer :: i
      character(len=12) :: bcname
      character(len=5),DIMENSION(8)  :: varext

      varext = (/'_BXS ', '_BTXS', '_BXE ', '_BTXE', '_BYS ', '_BTYS', '_BYE ', '_BTYE'/)


      do i = 1, 8
         bcname = trim(vname)//trim(varext(i))
         !     get variable id
         status = nf_inq_varid( ncid, trim(bcname), varid )
         if( status /= nf_noerr )  call handle_error( status )
         
         !     set start and count arrays
         nstt(1:4) = (/ 1, 1, 1, it /)
        
         if(i <= 4) ncnt(1:4) = (/ ny, nz, nw, 1 /)
         if(i >  4) ncnt(1:4) = (/ nx, nz, nw, 1 /)
         
         !     write BC values
         ! west
         if ( i == 1 ) then
            status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), valxs(:,:,:) )
             write(6,*)'tks write valxs value',it,maxval(valxs),minval(valxs),valxs(5,5,1)
            if( status /= nf_noerr ) call handle_error( status )     
            ! east       
         elseif ( i == 3 ) then
            status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), valxe(:,:,:) )
            if( status /= nf_noerr ) call handle_error( status )  
            ! north
         elseif ( i == 5 ) then
            status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), valys(:,:,:) )
            if( status /= nf_noerr ) call handle_error( status ) 
            ! south
         elseif ( i == 7 ) then
            status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), valye(:,:,:) )
            if( status /= nf_noerr ) call handle_error( status )
         else
!tks            if ( it >= 2 ) then
!tks               nstt(1:4) = (/ 1, 1, 1, it-1 /)
               nstt(1:4) = (/ 1, 1, 1, it /)         !tks
               ! Tendencies 
               ! west  
               if ( i == 2 ) then
                  status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), vatxs(:,:,:) )
                  if( status /= nf_noerr ) call handle_error( status )  
                  ! east        
               elseif ( i == 4 ) then
                  status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), vatxe(:,:,:) )
                  if( status /= nf_noerr ) call handle_error( status ) 
                  ! north
               elseif ( i == 6 ) then
                  status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), vatys(:,:,:) )
                  if( status /= nf_noerr ) call handle_error( status )  
                  ! south
               elseif ( i == 8 ) then
                  status = nf_put_vara_real( ncid, varid, nstt(:), ncnt(:), vatye(:,:,:) )
                  if( status /= nf_noerr ) call handle_error( status )
               end if
!tks            end if
         end if
         
      end do
      end subroutine wrfchem_write4d_bc
!---------------------------------------------------------------------------------------------------
!     write four-dimensional data for initial condition
!---------------------------------------------------------------------------------------------------

      subroutine wrfchem_write4d_ic( vname, val )

      implicit none

!     input arguments
      character (len=*), intent(in) :: vname
      real, intent(in)              :: val(nx,ny,nz)

!     local arguments
      integer :: status
      integer :: varid

!     get variable id
      status = nf_inq_varid( ncidi, trim(vname), varid )
      if( status /= nf_noerr )  call handle_error( status )

!     set start and count arrays
      ncnt = (/ nx, ny, nz, 1 /)
      nstt = (/ 1 , 1 , 1 , 1/)

      status = nf_put_vara_real( ncidi, varid, nstt, ncnt, val )
      if( status /= nf_noerr ) call handle_error( status )

      end subroutine wrfchem_write4d_ic


!---------------------------------------------------------------------------------------------------
!     handle errors produced by calling netCDF functions
!---------------------------------------------------------------------------------------------------

      subroutine handle_error( status )

      implicit none

!     input arguments :
      integer, intent(in) :: status

!     print the error information from processing NETcdf file
      print*, nf_strerror( status )

!     exit from the bconLib
      call exit_wrfchem_lib( flag=1 )

      end subroutine handle_error

!---------------------------------------------------------------------------------------------------
!     exit from bconLib
!---------------------------------------------------------------------------------------------------

      subroutine exit_wrfchem_lib( flag )

!     input arguments
      integer, optional, intent(in) :: flag

!     local arguments
      integer :: status

!     close netCDF file
      if( ncid /= 0 ) status = nf_close( ncid )
!     ajl add 1/13/2017 could have been a bug
      if( ncidi /= 0 ) status = nf_close( ncidi )

!     output information
      if( present(flag) ) then
        select case( flag )
          case( 1 ); print*, 'module_wrfchem_lib: fail to process netCDF file...'
          case( 2 ); print*, 'no such a species to save ...' 
          case default; print*, 'unknown error(s) occurred ...'
        endselect
        stop ' in moudle_wrfchem_lib ...'
      else
        print*, 'successfully exit from moudle_wrfchem_lib ...'
      endif 

      end subroutine exit_wrfchem_lib

      end module module_wrfchem_lib

!---------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------
